#include <stdio.h>

/* Shark SMI reg */
#if defined(MV_88W8660)
#define ETH_PHY_SMI_REG	0x78010

/* Orion SMI reg */
#elif defined(MV_88F5181L)
#define ETH_PHY_SMI_REG	0x72004

#else
#error "Undefined ETH_PHY_SMI_REG!"
#endif

/* SMI register fields (ETH_PHY_SMI_REG) */
#define ETH_PHY_SMI_DATA_OFFS 0
#define ETH_PHY_SMI_DATA_MASK (0xffff << ETH_PHY_SMI_DATA_OFFS)
#define ETH_PHY_SMI_DEV_ADDR_OFFS 16
#define ETH_PHY_SMI_DEV_ADDR_MASK (0x1f << ETH_PHY_SMI_DEV_ADDR_OFFS)
#define ETH_PHY_SMI_REG_ADDR_OFFS 21
#define ETH_PHY_SMI_REG_ADDR_MASK (0x1f << ETH_PHY_SMI_REG_ADDR_OFFS)	  
#define ETH_PHY_SMI_OPCODE_OFFS 26
#define ETH_PHY_SMI_OPCODE_MASK (3 << ETH_PHY_SMI_OPCODE_OFFS)
#define ETH_PHY_SMI_OPCODE_WRITE (0 << ETH_PHY_SMI_OPCODE_OFFS)
#define ETH_PHY_SMI_OPCODE_READ (1 << ETH_PHY_SMI_OPCODE_OFFS)
#define ETH_PHY_SMI_READ_VALID_BIT 27
#define ETH_PHY_SMI_READ_VALID_MASK (1 << ETH_PHY_SMI_READ_VALID_BIT)
#define ETH_PHY_SMI_BUSY_BIT 28
#define ETH_PHY_SMI_BUSY_MASK (1 << ETH_PHY_SMI_BUSY_BIT)  

#define ETH_PHY_TIMEOUT		10000

unsigned int mv_reg_read(unsigned int reg)
{
	unsigned int value;
	FILE *resource_dump;
	resource_dump = fopen ("/proc/resource_dump" , "w");
	if (!resource_dump) {
		printf ("Eror opening file /proc/resource_dump\n");
		exit(-1);
	}
	fprintf (resource_dump,"register  r %08x",reg);
	fclose (resource_dump);
	resource_dump = fopen ("/proc/resource_dump" , "r");
	if (!resource_dump) {
		printf ("Eror opening file /proc/resource_dump\n");
		exit(-1);
	}
	fscanf (resource_dump , "%x" , &value);
	fclose (resource_dump);
	return value;
}

void mv_reg_write(unsigned int reg, unsigned int value)
{
	FILE *resource_dump;
	resource_dump = fopen ("/proc/resource_dump" , "w");
	if (!resource_dump) {
		printf ("Eror opening file /proc/resource_dump\n");
		exit(-1);
	}
	fprintf (resource_dump,"register  w %08x %08x",reg,value);
	fclose (resource_dump);
	return;
}


void phy_reg_write(unsigned int phyAddr, unsigned int regOffs, short data)
{
	unsigned int smiReg;
	volatile unsigned int timeout;
	timeout = ETH_PHY_TIMEOUT;
	do
	{
		smiReg=mv_reg_read(ETH_PHY_SMI_REG);
		if (timeout-- == 0) {
			printf("ERROR: SMI busy timeout\n");
            		return;
		}
	}while (smiReg & ETH_PHY_SMI_BUSY_MASK);
	smiReg = (data << ETH_PHY_SMI_DATA_OFFS);
	smiReg |= (phyAddr <<  ETH_PHY_SMI_DEV_ADDR_OFFS) | (regOffs << ETH_PHY_SMI_REG_ADDR_OFFS );
    	smiReg &= ~ETH_PHY_SMI_OPCODE_READ;
	mv_reg_write(ETH_PHY_SMI_REG, smiReg);
	return;
}


void phy_reg_read(unsigned int phyAddr, unsigned int regOffs, short *data)
{
	unsigned int smiReg;
	volatile unsigned int timeout;
	timeout = ETH_PHY_TIMEOUT;
	do
	{
		smiReg = mv_reg_read(ETH_PHY_SMI_REG);
		if (timeout-- == 0) 
		{
			printf("phy_reg_read: SMI busy timeout\n");
            		return;
		}
	}while (smiReg & ETH_PHY_SMI_BUSY_MASK);
	smiReg = (phyAddr <<  ETH_PHY_SMI_DEV_ADDR_OFFS) | (regOffs << ETH_PHY_SMI_REG_ADDR_OFFS )|
			   ETH_PHY_SMI_OPCODE_READ;
	mv_reg_write(ETH_PHY_SMI_REG, smiReg);
	timeout=ETH_PHY_TIMEOUT;
	do
	{
		smiReg=mv_reg_read(ETH_PHY_SMI_REG);
		if (timeout-- == 0) {
			printf("ERROR: SMI read-valid timeout\n");
            		return;
		}
	}while (!(smiReg & ETH_PHY_SMI_READ_VALID_MASK));
	sleep(1);
	*data = (short)( mv_reg_read(ETH_PHY_SMI_REG) & ETH_PHY_SMI_DATA_MASK);
	return;
}

int main(int argc, char **argv)
{
    	unsigned int dev,reg,tmp;
	unsigned short data;
    	if ( (argc == 4) && (strcmp(argv[1],"-r")==0) ) {
        	sscanf(argv[2], "%x", &dev);
		sscanf(argv[3], "%x", &reg);
		phy_reg_read(dev, reg, &data);
	}
	else if ( (argc == 5) && (strcmp(argv[1],"-w")==0) ) {
        	sscanf(argv[2], "%x", &dev);
        	sscanf(argv[3], "%x", &reg);
		sscanf(argv[4], "%x", &tmp);
		data = tmp & 0xffff;
		phy_reg_write(dev, reg, data);
		phy_reg_read(dev, reg, &data);
	}
    	else { 
		printf("Usage:\n");
		printf("mv_smi_tool -r <smi_device> <reg_offs>\n");
		printf("mv_smi_tool -w <smi_device> <reg_offs> <data>\n");
        	return 0;
    	}
	printf("SMI Device 0x%x, Reg 0x%x, Data 0x%04x\n",dev,reg,data);
    	return 1;

}
